@@ -16,14 +16,18 @@
 #
 # SPDX-License-Identifier: Apache-2.0
 # Copyright (c) OWASP Foundation. All Rights Reserved.
+import warnings
 from datetime import datetime, timezone
-from typing import Iterable, Optional, Set
-from uuid import uuid4, UUID
+from typing import Iterable, Optional
+from uuid import UUID, uuid4
 
-from . import ExternalReference, OrganizationalContact, OrganizationalEntity, LicenseChoice, Property, ThisTool, Tool
+from sortedcontainers import SortedSet
+
+from ..exception.model import UnknownComponentDependencyException
+from ..parser import BaseParser
+from . import ExternalReference, LicenseChoice, OrganizationalContact, OrganizationalEntity, Property, ThisTool, Tool
 from .component import Component
 from .service import Service
-from ..parser import BaseParser
 
 
 class BomMetaData:
@@ -41,15 +45,15 @@ class BomMetaData:
                  licenses: Optional[Iterable[LicenseChoice]] = None,
                  properties: Optional[Iterable[Property]] = None) -> None:
         self.timestamp = datetime.now(tz=timezone.utc)
-        self.tools = set(tools or [])
-        self.authors = set(authors or [])
+        self.tools = tools or []  # type: ignore
+        self.authors = authors or []  # type: ignore
         self.component = component
         self.manufacture = manufacture
         self.supplier = supplier
-        self.licenses = set(licenses or [])
-        self.properties = set(properties or [])
+        self.licenses = licenses or []  # type: ignore
+        self.properties = properties or []  # type: ignore
 
-        if not self.tools:
+        if not tools:
             self.tools.add(ThisTool)
 
     @property
@@ -67,7 +71,7 @@ class BomMetaData:
         self._timestamp = timestamp
 
     @property
-    def tools(self) -> Set[Tool]:
+    def tools(self) -> "SortedSet[Tool]":
         """
         Tools used to create this BOM.
 
@@ -78,10 +82,10 @@ class BomMetaData:
 
     @tools.setter
     def tools(self, tools: Iterable[Tool]) -> None:
-        self._tools = set(tools)
+        self._tools = SortedSet(tools)
 
     @property
-    def authors(self) -> Set[OrganizationalContact]:
+    def authors(self) -> "SortedSet[OrganizationalContact]":
         """
         The person(s) who created the BOM.
 
@@ -96,7 +100,7 @@ class BomMetaData:
 
     @authors.setter
     def authors(self, authors: Iterable[OrganizationalContact]) -> None:
-        self._authors = set(authors)
+        self._authors = SortedSet(authors)
 
     @property
     def component(self) -> Optional[Component]:
@@ -153,7 +157,7 @@ class BomMetaData:
         self._supplier = supplier
 
     @property
-    def licenses(self) -> Set[LicenseChoice]:
+    def licenses(self) -> "SortedSet[LicenseChoice]":
         """
         A optional list of statements about how this BOM is licensed.
 
@@ -164,10 +168,10 @@ class BomMetaData:
 
     @licenses.setter
     def licenses(self, licenses: Iterable[LicenseChoice]) -> None:
-        self._licenses = set(licenses)
+        self._licenses = SortedSet(licenses)
 
     @property
-    def properties(self) -> Set[Property]:
+    def properties(self) -> "SortedSet[Property]":
         """
         Provides the ability to document properties in a key/value store. This provides flexibility to include data not
         officially supported in the standard without having to use additional namespaces or create extensions.
@@ -182,7 +186,7 @@ class BomMetaData:
 
     @properties.setter
     def properties(self, properties: Iterable[Property]) -> None:
-        self._properties = set(properties)
+        self._properties = SortedSet(properties)
 
     def __eq__(self, other: object) -> bool:
         if isinstance(other, BomMetaData):
@@ -235,9 +239,9 @@ class Bom:
         """
         self.uuid = uuid4()
         self.metadata = BomMetaData()
-        self.components = set(components or [])
-        self.services = set(services or [])
-        self.external_references = set(external_references or [])
+        self.components = components or []  # type: ignore
+        self.services = services or []  # type: ignore
+        self.external_references = external_references or []  # type: ignore
 
     @property
     def uuid(self) -> UUID:
@@ -271,7 +275,7 @@ class Bom:
         self._metadata = metadata
 
     @property
-    def components(self) -> Set[Component]:
+    def components(self) -> "SortedSet[Component]":
         """
         Get all the Components currently in this Bom.
 
@@ -282,7 +286,7 @@ class Bom:
 
     @components.setter
     def components(self, components: Iterable[Component]) -> None:
-        self._components = set(components)
+        self._components = SortedSet(components)
 
     def get_component_by_purl(self, purl: Optional[str]) -> Optional[Component]:
         """
@@ -325,7 +329,7 @@ class Bom:
         return component in self.components
 
     @property
-    def services(self) -> Set[Service]:
+    def services(self) -> "SortedSet[Service]":
         """
         Get all the Services currently in this Bom.
 
@@ -336,10 +340,10 @@ class Bom:
 
     @services.setter
     def services(self, services: Iterable[Service]) -> None:
-        self._services = set(services)
+        self._services = SortedSet(services)
 
     @property
-    def external_references(self) -> Set[ExternalReference]:
+    def external_references(self) -> "SortedSet[ExternalReference]":
         """
         Provides the ability to document external references related to the BOM or to the project the BOM describes.
 
@@ -350,7 +354,7 @@ class Bom:
 
     @external_references.setter
     def external_references(self, external_references: Iterable[ExternalReference]) -> None:
-        self._external_references = set(external_references)
+        self._external_references = SortedSet(external_references)
 
     def has_vulnerabilities(self) -> bool:
         """
@@ -362,6 +366,37 @@ class Bom:
         """
         return any(c.has_vulnerabilities() for c in self.components)
 
+    def validate(self) -> bool:
+        """
+        Perform data-model level validations to make sure we have some known data integrity prior to attempting output
+        of this `Bom`
+
+        Returns:
+             `bool`
+        """
+
+        # 1. Make sure dependencies are all in this Bom.
+        all_bom_refs = set([self.metadata.component.bom_ref] if self.metadata.component else []) | set(
+            map(lambda c: c.bom_ref, self.components)) | set(map(lambda s: s.bom_ref, self.services))
+
+        all_dependency_bom_refs = set().union(*(c.dependencies for c in self.components))
+        dependency_diff = all_dependency_bom_refs - all_bom_refs
+        if len(dependency_diff) > 0:
+            raise UnknownComponentDependencyException(
+                f'One or more Components have Dependency references to Components/Services that are not known in this '
+                f'BOM. They are: {dependency_diff}')
+
+        # 2. Dependencies should exist for the Component this BOM is describing, if one is set
+        if self.metadata.component and not self.metadata.component.dependencies:
+            warnings.warn(
+                f'The Component this BOM is describing {self.metadata.component.purl} has no defined dependencies'
+                f'which means the Dependency Graph is incomplete - you should add direct dependencies to this Component'
+                f'to complete the Dependency Graph data.',
+                UserWarning
+            )
+
+        return True
+
     def __eq__(self, other: object) -> bool:
         if isinstance(other, Bom):
             return hash(other) == hash(self)
